/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Kamen Lisp.
 *
 * The Initial Developer of the Original Code is
 * Dustin Long (dlong@stevens.edu).
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "marshalling.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

variantHolder makeSymbolVariant(char * data)
{
    variantHolder ret;
    ret._tag = variantHolder::VARIANT_SYMBOL;
    ret._raw = data;
    return ret;
}

variantHolder makeTextVariant(char * data)
{
    variantHolder ret;
    ret._tag = variantHolder::VARIANT_TEXT;
    ret._raw = data;
    return ret;
}

variantHolder makeAllocatedVariant(char * data)
{
    variantHolder ret;
    ret._tag = variantHolder::VARIANT_REPLY;
    ret._raw = data;
    return ret;
}

variantHolder makeIntVariant(int num)
{
    variantHolder ret;
    ret._tag = variantHolder::VARIANT_INT;
    ret._raw = (void*)num;
    return ret;
}

variantHolder makeDomVariant(void * data)
{
    variantHolder ret;
    ret._tag = variantHolder::VARIANT_DOM;
    ret._raw = data;
    return ret;
}

void * fetchVariantDom(variantHolder var)
{
    if (var._tag == variantHolder::VARIANT_DOM)
    {
        return var._raw;
    }
    return 0;
}

char * fetchVariantSymbol(variantHolder var)
{
    if (var._tag == variantHolder::VARIANT_SYMBOL)
    {
        return (char*)var._raw;
    }
    return 0;
}

char * fetchVariantText(variantHolder var)
{
    if ((var._tag == variantHolder::VARIANT_TEXT) ||
        (var._tag == variantHolder::VARIANT_REPLY))
    {
        return (char*)var._raw;
    }
    return 0;
}

int fetchVariantInt(variantHolder var, int * out)
{
    if (var._tag == variantHolder::VARIANT_INT)
    {
        *out = (int)var._raw;
        return 1;
    }
    return 0;
}

char * variantStringable(variantHolder * holder, variantHolder var)
{
    if ((var._tag == variantHolder::VARIANT_SYMBOL) ||
        (var._tag == variantHolder::VARIANT_TEXT) ||
        (var._tag == variantHolder::VARIANT_REPLY))
    {
        holder->_tag = variantHolder::VARIANT_TEXT;
        holder->_raw = (char*)var._raw;
        return (char*)holder->_raw;
    }
    else if (var._tag == variantHolder::VARIANT_INT)
    {
        size_t size = (sizeof(int) * 8 + 1);

        char * mine = (char*)malloc(size);
        sprintf(mine, "%d", var._raw);

        holder->_tag = variantHolder::VARIANT_REPLY;
        holder->_raw = (char*)mine;
        return mine;
    }
    else if (var._tag == variantHolder::VARIANT_DOM)
    {
        return 0;
    }
    return 0;
}

int variantEquals(variantHolder var, char * data)
{
    if ((var._tag == variantHolder::VARIANT_TEXT) ||
        (var._tag == variantHolder::VARIANT_SYMBOL) ||
        (var._tag == variantHolder::VARIANT_REPLY))
    {
        char * cmp = (char*)var._raw;
        if (strcmp(cmp, data) == 0)
        {
            return 1;
        }
    }
    return 0;
}

int variantNull(variantHolder var)
{
    return ((var._tag == 0) && (var._raw == 0));
}

dom_action_mode variantToActionMode(variantHolder var)
{
    if      (variantEquals(var, "DOM"))      return DOM_ACTION_DEFAULT;
    else if (variantEquals(var, "DOM-GET"))  return DOM_ACTION_GET;
    else if (variantEquals(var, "DOM-SET"))  return DOM_ACTION_SET;
    else if (variantEquals(var, "DOM-CALL")) return DOM_ACTION_CALL;
    return DOM_ACTION_UNKNOWN;
}

void destroyVariant(variantHolder * var)
{
    if (var->_tag == variantHolder::VARIANT_REPLY)
    {
        free(var->_raw);
    }
}
