/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Kamen Lisp.
 *
 * The Initial Developer of the Original Code is
 * Dustin Long (dlong@stevens.edu).
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _MARSHALLING_H
#define _MARSHALLING_H


// int is just a number
// symbol corresponds to a lisp symbol (uses static string data)
// text corresponds to a lisp string
// dom is an nsISupports pointer that has been AddRef'd
// reply is a malloc'd string, that will be freed when destroyed

struct variantHolder
{
    variantHolder() : _tag(0), _raw(0) {}
    int _tag;
    void * _raw;
    enum type { VARIANT_INT = 1, VARIANT_SYMBOL, VARIANT_TEXT, VARIANT_DOM, VARIANT_REPLY };
};

struct variantHolderArray
{
    int size;
    variantHolder * array;
};



variantHolder makeSymbolVariant(char * data);
variantHolder makeTextVariant(char * data);
variantHolder makeDomVariant(void * data);
variantHolder makeIntVariant(int num);
variantHolder makeAllocatedVariant(char * data);

char * fetchVariantSymbol(variantHolder var);
char * fetchVariantText(variantHolder var);
int fetchVariantInt(variantHolder var, int * out);
void * fetchVariantDom(variantHolder var);

char * variantStringable(variantHolder * holder, variantHolder var);
int variantEquals(variantHolder var, char * data);

int variantNull(variantHolder var);

void destroyVariant(variantHolder * holder);




enum dom_action_mode
{
    DOM_ACTION_UNKNOWN = 0,
    DOM_ACTION_DEFAULT,
    DOM_ACTION_GET,
    DOM_ACTION_SET,
    DOM_ACTION_CALL
};

dom_action_mode variantToActionMode(variantHolder var);





struct staticTable
{
    void * self;
    void (*domAction)(variantHolder * outc, void * self, variantHolderArray vars, int * err);
    void (*alertCall)(void * self, variantHolderArray vars);
    void (*finalizeDom)(void * self, void * data);
};


#define DOM_OBJECT_NAME "foreign_dom"

#ifndef _WIN32
#define _stdcall
#endif

typedef void (_stdcall * ResultProc)(const char *, int, void *);

typedef int (_stdcall * LispRoutine)(const char *, int, ResultProc, void *);

typedef int (_stdcall * StaticTableAcceptor)(staticTable * st);

#endif
