
#include <ecl/ecl.h>

extern int g_security_manager;

int g_security_manager = 0;

cl_object g_current_package = Cnil;


cl_object ext_call_secure(cl_object func);
cl_object ext_secure_safe_eval(cl_narg narg, cl_object form,
                               cl_object env, cl_object value);
int security_version();
cl_object security_failure(cl_object msg);
void security_check_finding(cl_object pack);
void security_check(char * text);
cl_object ext_security_check(cl_object msg);
void init_security();

int packageNamed(cl_object pack, cl_object want);


cl_object ext_call_secure(cl_object func)
{
	cl_object ret = Cnil;
	int preserve_security = g_security_manager;
	cl_object preserve_package = g_current_package;
	CL_UNWIND_PROTECT_BEGIN {
		g_security_manager = 1;
		g_current_package  = ecl_current_package();
		ret = cl_funcall(1, func);
	} CL_UNWIND_PROTECT_EXIT {
		g_security_manager = preserve_security;
		g_current_package  = preserve_package;
	} CL_UNWIND_PROTECT_END;
	return ret;
}

cl_object ext_secure_safe_eval(cl_narg narg, cl_object form,
                               cl_object env, cl_object value)
{
	cl_object ret = Cnil;
	int preserve_security = g_security_manager;
	cl_object preserve_package = g_current_package;
	CL_UNWIND_PROTECT_BEGIN {
		g_security_manager = 1;
		g_current_package  = ecl_current_package();
		ret = si_safe_eval(narg, form, env, value);
	} CL_UNWIND_PROTECT_EXIT {
		g_security_manager = preserve_security;
		g_current_package  = preserve_package;
	} CL_UNWIND_PROTECT_END;
	return ret;
}

int security_version()
{
	return 10101;
}

cl_object security_failure(cl_object msg)
{
	FEerror("Insecure operation: ~A", 1, msg);
	return Cnil;
}

void security_check_finding(cl_object pack)
{
	if (g_security_manager)
	{
		cl_object keyword, ffi;
		if (pack == g_current_package)
		{
			return;
		}
		if (packageNamed(pack, g_current_package->pack.name))
		{
			return;
		}
		keyword = make_simple_base_string("KEYWORD");
		if (packageNamed(pack, keyword))
		{
			return;
		}
		ffi = make_simple_base_string("FFI");
		if (packageNamed(pack, ffi))
		{
			return;
		}
		FEerror("Insecure operation: Finding package ~A", 1, pack);
	}
}

/* Precondition: 'want' is a base_string */
int packageNamed(cl_object pack, cl_object want)
{
	cl_object name;
	if (type_of(pack) == t_package)
	{
		name = pack->pack.name;
	}
	else if (type_of(pack) == t_base_string)
	{
		name = pack;
	}
	else
	{
		return 0;
	}
	if (strcmp(name->base_string.self, want->base_string.self) == 0)
	{
		return 1;
	}
	return 0;
}

void security_check(char * text)
{
	if (g_security_manager)
	{
		cl_object msg = make_simple_base_string(text);
		security_failure(msg);        
	}
}

cl_object ext_security_check(cl_object msg)
{
	if (g_security_manager)
	{
		return security_failure(msg);
	}
	return Cnil;
}

void init_security()
{
	cl_object callSecureText = make_simple_base_string("CALL-SECURE");
	cl_object secureCheckText = make_simple_base_string("SECURITY-CHECK");

	cl_object extText = make_simple_base_string("EXT");
	cl_object package = cl_find_package(extText);

	cl_object callSecureSym, secureCheckSym;

	callSecureSym = cl_intern(2, callSecureText, extText);
	cl_def_c_function(callSecureSym,
	                 (void*)ext_call_secure, 1);
	cl_export2(callSecureSym, package);

	secureCheckSym = cl_intern(2, secureCheckText, extText);
	cl_def_c_function(secureCheckSym,
	                  (void*)ext_security_check, 1);
	cl_export2(secureCheckSym, package);
}

