/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Kamen Lisp.
 *
 * The Initial Developer of the Original Code is
 * Dustin Long (dlong@stevens.edu).
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var KamenLispWebProgressListener =
{
    STATE_START : Components.interfaces.nsIWebProgressListener.STATE_START,

    STATE_STOP : Components.interfaces.nsIWebProgressListener.STATE_STOP,

    QueryInterface: function(aIID)
    {
        if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
            aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
            aIID.equals(Components.interfaces.nsISupports))
            return this;
        throw Components.results.NS_NOINTERFACE;
    },
    
    onStateChange: function(aProgress, aRequest, aFlag, aStatus)
    {
        if(aFlag & this.STATE_START)
        {

        }
        if(aFlag & this.STATE_STOP)
        {
            var window = null;
            var obj = aProgress.DOMWindow;
            if (!obj || !obj.wrappedJSObject)
            {
                return false;
            }

            window = obj.wrappedJSObject;
            var doc = window.document;
            var url = doc.location.toString();

            if (!KamenLispGlobal.hasCLScriptTags(doc))
            {
                return false;
            }
            if (!KamenLispPermissions.allowed(url))
            {
                return false;
            }

            window.addEventListener("unload",
                                    function () {
                                        KamenLispGlobal.onPageUnload(doc);
                                    }, false);

            setTimeout(function() {
                           KamenLispGlobal.createIsolationPackage(doc);
                           KamenLispGlobal.discoverLispCallers(doc);
                           KamenLispGlobal.runScriptTagsInPage(doc);
                       }, 0);
        }
        return 0;
    },

    onLocationChange: function(aProgress, aRequest, aURI)
    {
        return 0;
    },

    onProgressChange: function() {return 0;},
    onStatusChange: function() {return 0;},
    onSecurityChange: function() {return 0;},
    onLinkIconAvailable: function() {return 0;}
}

var KamenLispGlobal =
{
    callThisPage : function()
    {
        var win = KamenLispUtils.getHtmlWindow();
        this.runScriptTagsInPage(win.document);
    },

    s_lispRunner : null,

    runLispCode : function(doc, code)
    {
        var runner = this.ensureLispRunner();
        var win = doc.defaultView;
        var outc = new Object();
        var res = runner.topWindow(win);

        var pack = this.fetchPackage(doc);
        runner.usePackage("(use)", pack, outc);

        res = runner.evaluate(code, outc);
        if (res == 1)
        {
            throw outc.value;
        }
        return outc.value;
    },

    invokeLispFunction : function(func)
    {
        var err = null;
        try
        {
            var runner = this.ensureLispRunner();
            var win = KamenLispUtils.getHtmlWindow();
            var outc = new Object();
            var res = runner.topWindow(win);

            //runner.evaluate("nil", outc);

            var pack = this.fetchPackage(win.document);
            runner.usePackage("(use)", pack, outc);

            res = runner.invoke(func, outc);
            if (res == 1)
            {
                err = outc.value;
                dump("set err to [" + err + "]\n");
            }

            if (res == 0)
            {
                return true;
            }
        }
        catch (e)
        {
            dump("error: [" + e + "]\n");
        }

        if (err != null)
        {
            dump("throwing [" + err + "]\n");
            throw err;
        }

        return false;
    },


    savePackage : function(doc, packageName)
    {
        var div = doc.createElement("div");
        div.setAttribute("name", "package");
        div.style.visibility = "hidden";
        div.textContent = packageName;
        doc.firstChild.appendChild(div);
    },

    fetchPackage : function(doc)
    {
        var child = doc.firstChild.lastChild;
        var packageName = child.textContent;
        if (!packageName)
        {
            throw "No package";
        }
        // package must be generated symbol
        if (packageName.charAt(0) != "G")
        {
            throw "Illegal package";
        }
        return packageName;
    },

    ensureLispRunner : function()
    {
        if (!this.s_lispRunner)
        {
            try
            {
                var obj = Components.classes["@stevens.edu/~dlong/KamenLisp/KamenRettsuGo;1"].createInstance();
                obj = obj.QueryInterface(Components.interfaces.IKamenLisper);
            } catch (err) {
                alert(err);
                return null;
            }

            var status = obj.boot();
            if (status != 0)
            {
                dump("Booting failed with error [" + status + "]\n");
            }

            this.s_lispRunner = obj;        
        }
        
        return this.s_lispRunner;
    },

    discoverLispCallers : function(doc)
    {
        var xpath = "//*[@onclick|@onmouseover|@onfocus|@onload]";
        var res = doc.evaluate(xpath, doc, null, XPathResult.ANY_TYPE, null);

        var elems = new Array;

        for (;;)
        {
            var responder = res.iterateNext();
            if (!responder) { break; }
            elems.push(responder);
        }

        for (var i = 0; i < elems.length; i++)
        {
            KamenLispGlobal.transformLispCallers(elems[i]);
        }
    },

    transformLispCallers : function(elem)
    {
        var attrs = elem.attributes;
        for (var i = 0; i < attrs.length; i++)
        {
            var attr = attrs.item(i);
            var val = attr.nodeValue;
            if (val[0] == "(" &&
                val[val.length - 1] == ")")
            {
                var call = val.substring(1, val.length - 1);

                elem[attr.nodeName] = function() {
                    var element = document.createElement("LispInvokeElem");
                    element.setAttribute("call", call);
                    document.documentElement.appendChild(element);
                    var event = document.createEvent("Events");
                    event.initEvent("LispInvocation", true, false);
                    element.dispatchEvent(event);
                };
            }
        }
    },

    runScriptTagsInPage : function(doc)
    {
        var elems = doc.getElementsByTagName("script");
        var pageHasLisp = false;
        for (var k = 0; k < elems.length; k++)
        {
            var language = elems[k].getAttribute("language");
            if (language == "text/common-lisp")
            {
                var got = this.runLispCode(doc, elems[k].textContent);
                pageHasLisp = true;
            }
        }
    },

    // refactor this with runScriptTags
    hasCLScriptTags : function(doc)
    {
        var elems = doc.getElementsByTagName("script");
        var pageHasLisp = false;
        for (var k = 0; k < elems.length; k++)
        {
            var language = elems[k].getAttribute("language");
            if (language == "text/common-lisp")
            {
                return true;
            }
        }
        return false;
    },

    createIsolationPackage : function(doc)
    {
        var runner = this.ensureLispRunner();
        var outc = new Object();
        runner.usePackage("(new)", "", outc);
        var pack = outc.value;
        this.savePackage(doc, pack);
    },

    onPageLoad : function()
    {
        gBrowser.addProgressListener(KamenLispWebProgressListener,
                                     Components.interfaces
                                     .nsIWebProgress.NOTIFY_STATE_DOCUMENT);
    },

    onPageUnload : function(document)
    {
        var pack = this.fetchPackage(document);
        var runner = this.ensureLispRunner();
        var outc = new Object();
        runner.usePackage("(release)", pack, outc);
    },

    onAppcontentLoaded : function()
    {
    },

    config : function(url)
    {
        window.openDialog("chrome://kamen/content/config.xul",
                          "_blank",
                          "chrome,dialog=yes,modal=yes,centerscreen=yes",
                          {defaultUrl: url, permObj: KamenLispPermissions});
    }
};

window.addEventListener("load",   KamenLispGlobal.onPageLoad, false);

window.addEventListener("LispInvocation", function(e) {
    var call = e.target.getAttribute("call");
    KamenLispGlobal.invokeLispFunction(call);
}, false, true);

