/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Kamen Lisp.
 *
 * The Initial Developer of the Original Code is
 * Dustin Long (dlong@stevens.edu).
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var KamenLispPermissions =
{
    m_whiteList : null,
    
    load : function()
    {
    this.m_whiteList = new Array();

    var text = "";

    var file = this.getDataFile("whitelist.xml");
    if (!file.exists())
    {
        text = "<WhiteList><site>" +
               "<domain>cs.stevens.edu</domain>" +
               "<path>/~dlong</path>" +
               "</site></WhiteList>";
        //dump("file not exist\n");
    }
    else
    {
        var inStream = this.getReadStream(this.getDataFile("whitelist.xml"));
        text = this.readContents(inStream);
        //dump("file loaded\n");
    }

    var domParser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
                    .createInstance(Components.interfaces.nsIDOMParser);
    var doc = domParser.parseFromString(text, "text/xml");
    var nodes = doc.evaluate("/WhiteList/site", doc, null, 0, null);
    var node;

    while ((node = nodes.iterateNext()))
    {
        var allow = new Object();
        for (var k = 0; k < node.childNodes.length; k++)
        {
        var child = node.childNodes[k];

        var attr = child.nodeName;
        var val = child.firstChild.nodeValue;

        allow[attr] = val;
        }
        this.m_whiteList.push(allow);
    }

        //dump("load whitelist length [" + this.m_whiteList.length + "]\n");
    },

    m_isInitialized : false,

    ensure : function()
    {
    if (!this.m_isInitialized)
    {
        this.load();
        this.m_isInitialized = true;
    }
    },

    fetch : function()
    {
    this.ensure();
    return this.m_whiteList;
    },

    display : function()
    {
    this.ensure();
    dump("=====================\n");
    for (var i = 0; i < this.m_whiteList.length; i++)
    {
        var allow = this.m_whiteList[i];
        dump("[" + allow.domain + "] [" + allow.path + "]\n");
    }
    dump("=====================\n");
    dump("\n");
    },

    update : function(ls)
    {
    this.m_whiteList = new Array();

    var doc = document.implementation.createDocument("", "WhiteList", null);

    for (var i = 0; i < ls.length; i++)
    {
        var siteNode = doc.createElement("site");
        var parts = this.split(ls[i]);
        var domainNode = doc.createElement("domain");
        var domain = doc.createTextNode(parts.domain);
        domainNode.appendChild(domain);
        siteNode.appendChild(domainNode);
        var pathNode = doc.createElement("path");
        var path = doc.createTextNode(parts.path);
        pathNode.appendChild(path);
        siteNode.appendChild(pathNode);

        doc.firstChild.appendChild(doc.createTextNode("\n\t"));
        doc.firstChild.appendChild(siteNode);

        this.m_whiteList.push(parts);
    }
    doc.firstChild.appendChild(doc.createTextNode("\n\t"));

        //dump("update whitelist length [" + this.m_whiteList.length + "]\n");

    var outStream = this.getWriteStream(this.getDataFile("whitelist.xml"));
    new XMLSerializer().serializeToStream(doc, outStream, "utf-8");
    outStream.close();
    },

    getDataFile : function(name)
    {
    var file = this.getDataDir();
    file.append(name);
    return file;
    },

    getDataDir : function()
    {
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                   .getService(Components.interfaces.nsIProperties)
                   .get("ProfD", Components.interfaces.nsILocalFile);
    file.append("kamen");
    if (!file.exists())
    {
        file.create(Components.interfaces.nsILocalFile.DIRECTORY_TYPE,
                0777);
    }
    return file;
    },

    getReadStream : function(file)
    {
    var stream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                               .createInstance(Components.interfaces.nsIFileInputStream);
    stream.init(file, 0x01 /*PR_RDONLY*/, 0660, 4 /*CLOSE_ON_EOF*/);
    return stream;
    },

    readContents : function(stream)
    {
    var scriptable = Components.classes["@mozilla.org/scriptableinputstream;1"]
                                   .getService(Components.interfaces.nsIScriptableInputStream);
    scriptable.init(stream);
    var contents = scriptable.read(scriptable.available());
    return contents;
    },

    getWriteStream : function(file)
    {
    var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                               .createInstance(Components.interfaces.nsIFileOutputStream);
    stream.init(file, 0x02 /*PR_WRONLY*/ | 0x08 /*PR_CREATE_FILE*/ | 0x20 /*PR_TRUNCATE*/, 0660, 0);
    return stream;
    },

    entry : function(text)
    {
    var parts = this.split(text);
    return parts.domain + parts.path;
    },

    split : function(text)
    {
    var start = 0;
    var domain = "";
    var path = "";

    if (text.substring(0, 7) == "http://")
    {
        start = 7;
    }

    var pos = text.indexOf("/", start);
    if (pos == -1)
    {
        domain = text.substring(start);
    }
    else
    {
        domain = text.substring(start, pos);
        path = text.substring(pos);
    }

    var ret = new Object();
    ret.domain = domain;
    ret.path = path;
    return ret;
    },

    whitelisted : function(url)
    {
    var allowed = this.fetch();
    var mine = this.split(url);
    for (var i = 0; i < allowed.length; i++)
    {
        if ((KamenLispUtils.endsWith(mine.domain, allowed[i].domain)) &&
        (KamenLispUtils.beginsWith(mine.path, allowed[i].path)))
        {
            //dump("match [" + allowed[i].domain + "] [" + allowed[i].path + "]\n");

        return true;
        }
    }
        //dump("no match\n");

    return false;
    },

    allowed : function(url)
    {
        //dump("should i allow [" + url + "]?\n");
    if (url.substring(0, 7) == "file://")
    {
        //dump("file yes!\n");
        return true;
    }
    else if (url.substring(0, 7) == "http://")
    {
        if (this.whitelisted(url))
        {
        //dump("white listed yes!\n");
        return true;
        }
        else
        {
        KamenLispShowMsg.display();
        return false;
        }
        //dump("http no!\n");
        return false;
    }
    return false;
    }
}
