/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Kamen Lisp.
 *
 * The Initial Developer of the Original Code is
 * Dustin Long (dlong@stevens.edu).
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _KAMEN_RETTSUGO_CPP
#define _KAMEN_RETTSUGO_CPP

#include "KamenRettsuGo.h"
#include "KamenLispExt.h"
#include "../../common/constants.h"

#ifdef _WIN32
#include <windows.h>
#else
#include <dlfcn.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "nsIExtensionManager.h"
#include "nsServiceManagerUtils.h"

#include "nsComponentManagerUtils.h"

#include "nsIDOMXULDocument.h"
#include "nsIDOMXULCommandDispatcher.h"
#include "nsIDOMWindow.h"
#include "nsIDOMDocument.h"

#include "nsIWindowMediator.h"
#include "nsIDOMWindowInternal.h"
#include "nsIDOMElement.h"
#include "nsIPromptService.h"
#include "nsIPrefBranch.h"
#include "nsIDOMHTMLInputElement.h"
#include "nsIDOMNSHTMLElement.h"

#include "nsIErrorService.h"


NS_IMPL_ISUPPORTS1(KamenRettsuGo, IKamenLisper)

KamenRettsuGo::KamenRettsuGo()
{
    this->m_embed = 0;
    this->m_invoke = 0;
    this->initStaticTable();
    this->m_saru.init();
}

KamenRettsuGo::~KamenRettsuGo()
{
}

void KamenRettsuGo::initStaticTable()
{
    m_table.self = this;
    m_table.domAction = KamenRettsuGo::domActionEntry;
    m_table.alertCall = KamenRettsuGo::alertCallEntry;
    m_table.finalizeDom = KamenRettsuGo::finalizeDomEntry;
}

void KamenRettsuGo::domActionEntry(variantHolder * outc, void * self,
                                   variantHolderArray vars, int * err)
{
    KamenRettsuGo * mine = (KamenRettsuGo*)self;
    *outc = mine->doDomAction(vars, err);
}

void KamenRettsuGo::alertCallEntry(void * self, variantHolderArray vars)
{
    KamenRettsuGo * mine = (KamenRettsuGo*)self;
    mine->doAlertCall(vars);
}

void KamenRettsuGo::finalizeDomEntry(void * self, void * data)
{
    nsISupports * supports = (nsISupports*)data;
    supports->Release();
}


#define CONTRACT_PREF   "@mozilla.org/preferences;1"
#define CONTRACT_EXTN   "@mozilla.org/extensions/manager;1"
#define CONTRACT_PROMPT "@mozilla.org/embedcomp/prompt-service;1"


#define DEBUG_LISP 1
#define DEBUG_DOM  2

bool KamenRettsuGo::isDebugging(int catagory)
{
    nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(CONTRACT_PREF));
    if (!prefBranch)
    {
        printf("getPref failed getting branch\n");
        return false;
    }

    PRBool ret = 0;

    nsresult rv;

    if (catagory == DEBUG_LISP)
    {
        rv = prefBranch->GetBoolPref("kamenlisp.debug.lisp", &ret);
        if (NS_FAILED(rv))
        {
            printf("getPref kamenlisp.debug.lisp failed\n");
            return false;
        }
        return ret;
    }
    else if (catagory == DEBUG_DOM)
    {
        rv = prefBranch->GetBoolPref("kamenlisp.debug.dom", &ret);
        if (NS_FAILED(rv))
        {
            printf("getPref kamenlisp.debug.dom failed\n");
            return false;
        }
        return ret;
    }
    else
    {
        printf("getPref unknown catagory %d\n", catagory);
        return false;
    }
}



variantHolder KamenRettsuGo::doDomAction(variantHolderArray vars, int * err)
{
    variantHolder outc;

    for (;;)
    {
        nsISupports * mustRelease = 0;

        if (vars.size <= SARU_DOM_OFFSET)   { *err = DOM_ERR_LACK ; break; }
        variantHolder dom = vars.array[SARU_DOM_OFFSET];

        dom_action_mode mode = variantToActionMode(dom);
        if (mode == DOM_ACTION_UNKNOWN)     { *err = DOM_ERR_FATAL; break; }

        if (vars.size <= SARU_HOST_OFFSET)  { *err = DOM_ERR_LACK ; break; }
        variantHolder host = vars.array[SARU_HOST_OFFSET];

        if (vars.size <= SARU_FIELD_OFFSET) { *err = DOM_ERR_LACK; break; }
        variantHolder field = vars.array[SARU_FIELD_OFFSET];

        variantHolderArray params;
        params.size = vars.size - SARU_PARAMS_OFFSET;
        params.array = vars.array + SARU_PARAMS_OFFSET;

        if ((mode == DOM_ACTION_GET) && (params.size != 0))
        {
            *err = DOM_ERR_EXTRA; break;
        }
        if ((mode == DOM_ACTION_SET) && (params.size < 1))
        {
            *err = DOM_ERR_LACK; break;
        }
        if ((mode == DOM_ACTION_SET) && (params.size > 1))
        {
            *err = DOM_ERR_EXTRA; break;
        }

        // TODO what about function calls that want to pass document or window?

        if (variantEquals(host, "DOCUMENT"))
        {
            nsresult rv;
            nsIDOMWindow * win = m_window;

            nsCOMPtr<nsIDOMDocument> docHtml;
            rv = win->GetDocument(getter_AddRefs(docHtml));
            if (!NS_SUCCEEDED(rv))
            {
                *err = DOM_ERR_FATAL; break;
            }

            nsCOMPtr<nsISupports> supports = do_QueryInterface(docHtml);
            mustRelease = supports;
            mustRelease->AddRef();

            host = makeDomVariant(mustRelease);
        }
        else if (variantEquals(host, "WINDOW"))
        {
            nsresult rv;
            nsCOMPtr<nsIDOMWindow> win = m_window;

            nsCOMPtr<nsISupports> supports = do_QueryInterface(win);
            mustRelease = supports;
            mustRelease->AddRef();

            host = makeDomVariant(mustRelease);
        }

        outc = m_saru.domAction(mode, host, field, params, err);

        if (mustRelease)
        {
            mustRelease->Release();
        }

        break;
    }

    return outc;
}


void KamenRettsuGo::doAlertCall(variantHolderArray vars)
{
    variantHolder var = vars.array[0];
    variantHolder holder;
    char * message = variantStringable(&holder, var);
    if (message)
    {
        nsresult rv;
        nsCOMPtr<nsIPromptService> promptSrv(do_GetService(CONTRACT_PROMPT));
        if (!promptSrv) return;

        promptSrv->Alert(m_window,
                         NS_ConvertUTF8toUTF16("Alert").get(),
                         NS_ConvertUTF8toUTF16(message).get());
    }
    destroyVariant(&holder);
}



nsISupports * variantToSupports(variantHolder var)
{
    void * val = fetchVariantDom(var);
    if (val)
    {
        return (nsISupports*)val;
    }
    return 0;
}



NS_IMETHODIMP KamenRettsuGo::TopWindow(nsIDOMWindow * window, PRInt16 *retval)
{
    this->m_window = window;

    *retval = 1;
    return NS_OK;
}

NS_IMETHODIMP KamenRettsuGo::Boot(PRInt16 *retval)
{
    *retval = -1;

    // cannot dual-boot
    if (this->isLispEmbedded())
    {
        return NS_ERROR_FAILURE;
    }

    // zero means success
    int ret = this->embedLisp();
    *retval = ret;
    return NS_OK;
}

NS_IMETHODIMP KamenRettsuGo::Evaluate(const nsACString & code,
                                      nsACString & outc,
                                      PRInt16 * retval)
{
    const char * codeText = 0;
    PRBool termin;
    NS_CStringGetData(code, &codeText, &termin);

    if (this->isDebugging(DEBUG_LISP))
    {
        printf("= Evaluate Lisp =\n"
               "%s\n"
               "=---------------=\n", codeText);
    }
    if (!this->runLisp(codeText, 1, &outc, retval))
    {
        return NS_ERROR_FAILURE;
    }
    if (this->isDebugging(DEBUG_LISP))
    {
        const char * cstr = 0;
        PRBool termin;
        NS_CStringGetData(outc, &cstr, &termin);
        printf("Got: [%s]\n\n", cstr);
    }
    return NS_OK;
}

NS_IMETHODIMP KamenRettsuGo::Invoke(const nsACString & code,
                                    nsACString & outc,
                                    PRInt16 * retval)
{
    const char * funcName = 0;
    PRBool termin;
    NS_CStringGetData(code, &funcName, &termin);

    if (this->isDebugging(DEBUG_LISP))
    {
        printf("= Invoke Function =\n"
               "%s\n"
               "=-----------------=\n", funcName);
    }
    if (!this->runLisp(funcName, 0, &outc, retval))
    {
        return NS_ERROR_FAILURE;
    }
    return NS_OK;
}

NS_IMETHODIMP KamenRettsuGo::UsePackage(const nsACString & command,
                                        const nsACString & package,
                                        nsACString & outc,
                                        PRInt16 * retval)
{
    const char * cmd = 0;
    PRBool termin;
    NS_CStringGetData(command, &cmd, &termin);
    
    const char * pName = 0;
    NS_CStringGetData(package, &pName, &termin);

    PRInt16 err = 0;

    if (strcmp(cmd, "(new)") == 0)
    {
        // Create new package
        runLisp("(let ((sym (gensym)))"
                "  (setf *package* (make-package sym)) sym)",
                200, &outc, &err);
        runLisp("(use-package 'kamen-common)",
                200, NULL, &err);
    }
    else if (strcmp(cmd, "(release)") == 0)
    {
        // Release finished package
        runLisp("(setf *package* (find-package 'kamen-common))",
                200, NULL, &err);

        char packCode[1024];
        sprintf(packCode, "(delete-package '#:%s))", pName);
        runLisp(packCode, 200, NULL, &err);
    }
    else if (strcmp(cmd, "(use)") == 0)
    {
        // Use existing package
        char packCode[1024];
        sprintf(packCode, "(setf *package* (find-package '#:%s))", pName);
        runLisp(packCode, 200, NULL, &err);
    }
    else
    {
        *retval = -1;
        return NS_ERROR_NOT_IMPLEMENTED;
    }

    *retval = 0;
    return NS_OK;
}


int KamenRettsuGo::runLisp(const char * sLisp, int codeType,
                           nsACString * outPlace, PRInt16 * retval)
{
    *retval = 0;

    if (!this->isLispEmbedded())
    {
        return 0;
    }

    lispRetResult result;
    result.text = outPlace;
    result.err = 0;

    LispRoutine routine = this->m_invoke;

    routine(sLisp,
            codeType,
            (ResultProc)KamenRettsuGo::lispResult,
            (void*)&result);

    *retval = result.err;

    return 1;
}

int KamenRettsuGo::isLispEmbedded() const
{
    return this->m_embed;
}

void KamenRettsuGo::lispResult(const char * got, int isErr, void * context)
{
    lispRetResult * result = (lispRetResult*)context;
    if (result->text)
    {
        int len = strlen(got);
        NS_CStringSetData(*(result->text), got, len);
    }
    result->err = isErr;
}



#ifdef _WIN32
const char * g_henshin_component = "henshin.dll";
#else
const char * g_henshin_component = "henshin.so";
#endif

int KamenRettsuGo::embedLisp()
{
    nsresult rv;
    nsCOMPtr<nsIExtensionManager> extMgr(do_GetService(CONTRACT_EXTN));
    if (!extMgr) return 101;

    nsEmbedCString id(KAMEN_LISP_EXT_ID);
    nsCOMPtr<nsIInstallLocation> installLocation;
    rv = extMgr->GetInstallLocation(NS_ConvertUTF8toUTF16(id),
                                    getter_AddRefs(installLocation));
    if (!NS_SUCCEEDED(rv)) return 201;
    if (!installLocation) return 202;

    nsCOMPtr<nsIFile> itemLocation;
    rv = installLocation->GetItemLocation(NS_ConvertUTF8toUTF16(id),
                                          getter_AddRefs(itemLocation));
    if (!NS_SUCCEEDED(rv)) return 203;

    nsEmbedCString comps("components");
    rv = itemLocation->AppendNative(comps);
    if (!NS_SUCCEEDED(rv)) return 204;

    nsEmbedCString hensh(g_henshin_component);
    rv = itemLocation->AppendNative(hensh);
    if (!NS_SUCCEEDED(rv)) return 205;

    PRBool exist;
    rv = itemLocation->Exists(&exist);
    if (!NS_SUCCEEDED(rv)) return 301;

    if (exist)
    {
	int ret = this->fetchLispRoutine(itemLocation);
        if (ret == 0)
        {
	    this->m_embed = 1;
        }
	return ret;
    }
    else
    {
	return 302;
    }
}


#ifdef _WIN32
const char * g_ecl_module = "ecl.dll";
#else
const char * g_ecl_module = "libecl.so";
#endif

// os specific code
int KamenRettsuGo::fetchLispRoutine(nsCOMPtr<nsIFile> & location)
{
  //printf("fetching lisp routine\n");

    nsresult rv;

    nsEmbedCString extensionPath;
    rv = location->GetNativePath(extensionPath);
    if (rv != NS_OK) { return 601; }

    const char * cstr = 0;
    PRBool termin;
    NS_CStringGetData(extensionPath, &cstr, &termin);

    if (!termin) { return 602; }

#ifdef _WIN32

    HMODULE hMod = ::LoadLibraryEx(cstr, NULL, 8);
    if (!hMod) { return 801; }

    FARPROC targetProc;

    targetProc = ::GetProcAddress(hMod, "evaluateLisp");
    if (!targetProc) { return 802; }
    LispRoutine lispProc = (LispRoutine)targetProc;

    targetProc = ::GetProcAddress(hMod, "setStaticTable");
    if (!targetProc) { return 803; }
    StaticTableAcceptor staticProc = (StaticTableAcceptor)targetProc;
    staticProc(&m_table);

    this->m_invoke = lispProc;
    return 0;

#else

    //printf("got henshin path\n");
    nsCOMPtr<nsIFile> depends;
    location->GetParent(getter_AddRefs(depends));
    if (!depends) return 1001;

    nsEmbedCString app(g_ecl_module);
    rv = depends->AppendNative(app);
    if (!NS_SUCCEEDED(rv)) return 1002;

    nsEmbedCString eclPath;
    rv = depends->GetNativePath(eclPath);
    if (rv != NS_OK) return 1003;

    //printf("got ecl path\n");

    const char * loc = 0;
    PRBool ends;
    NS_CStringGetData(eclPath, &loc, &ends);
    if (!ends) return 1004;

    printf("got string for ecl path [%s]\n", loc);
    void * lisp = dlopen(loc, RTLD_NOW | RTLD_GLOBAL);
    if (!lisp)
    {
	char * errmsg = dlerror();
	printf("Error loading libecl [%s]\n", errmsg);
	return 1005;
    }

    //if (lisp) { dlclose(lisp); }
    //printf("loaded ecl [%x]\n", lisp);

    printf("got string for henshin path [%s]\n", cstr);
    void * hMod = dlopen(cstr, RTLD_LAZY);
    if (!hMod) return 1006;

    //printf("opended henshin\n");
    void * targetProc;

    targetProc = dlsym(hMod, "evaluateLisp");
    if (!targetProc) return 1007;
    LispRoutine lispProc = (LispRoutine)targetProc;
    //printf("got lisp proc [%x]\n", lispProc);
    if (!lispProc) return 1008;

    targetProc = dlsym(hMod, "setStaticTable");
    if (!targetProc) return 1009;
    StaticTableAcceptor staticProc = (StaticTableAcceptor)targetProc;

    //printf("got static table setter [%x]\n", staticProc);
    staticProc(&m_table);
    //printf("called static table setter\n");

    //dlclose(hMod);

    this->m_invoke = lispProc;
    return 0;

#endif

}



#endif

