/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Kamen Lisp.
 *
 * The Initial Developer of the Original Code is
 * Dustin Long (dlong@stevens.edu).
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _KAMEN_TOOL_H_
#define _KAMEN_TOOL_H_

#include "IKamenLisper.h"

// {F310629E-2FEB-4ad6-8CB1-C3AC9C986B80}
//static const GUID <<name>> =
//{ 0xf310629e, 0x2feb, 0x4ad6, { 0x8c, 0xb1, 0xc3, 0xac, 0x9c, 0x98, 0x6b, 0x80 } };


#include "../../common/marshalling.h"



#define KAMEN_RETTSUGO_CONTRACTID "@stevens.edu/~dlong/KamenLisp/KamenRettsuGo;1"
#define KAMEN_RETTSUGO_CLASSNAME "Kamen Lisp Host"
#define KAMEN_RETTSUGO_CID { 0xf310629e, 0x2feb, 0x4ad6, { 0x8c, 0xb1, 0xc3, 0xac, 0x9c, 0x98, 0x6b, 0x80 } }



#include "nsCOMPtr.h"
#include "nsIFile.h"
#include "nsStringAPI.h"
#include "nsEmbedString.h"

#include "nsIDOMWindow.h"

#include "SaruNoKumo.h"



struct lispRetResult
{
    nsACString * text;
    int err;
};


class KamenRettsuGo : public IKamenLisper
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_IKAMENLISPER

  KamenRettsuGo();

private:
  ~KamenRettsuGo();
  int m_embed;
  LispRoutine m_invoke;
  nsCOMPtr<nsIDOMWindow> m_window;
  nsCOMPtr<nsIDOMElement> m_element2;
  nsCOMPtr<nsIDOMElement> m_element3;

  void initStaticTable();

  staticTable m_table;
  static void domActionEntry(variantHolder * outc, void * self,
                             variantHolderArray vars, int * err);
  variantHolder doDomAction(variantHolderArray vars, int * err);
  static void alertCallEntry(void * self, variantHolderArray vars);
  void doAlertCall(variantHolderArray vars);
  void docGetElementById(variantHolder spec, nsISupports ** supports);
  static void finalizeDomEntry(void * self, void * data);

  bool isDebugging(int catagory);

protected:

  int runLisp(const char * sLisp, int isCode,
              nsACString * outPlace, PRInt16 * retval);

  int isLispEmbedded() const;
  int embedLisp();

  int fetchLispRoutine(nsCOMPtr<nsIFile> & location);

  static void lispResult(const char * got, int isErr, void * context);
  int loadDynamicParts(nsIFile * itemLocation);

  SaruNoKumo m_saru;

};


#endif
